==> mkreadmes-1.2.tar.bz2: mkreadmes-1.2/util.c <==
/*
 *	util.c
 *
 *	utility routines called from a number of different places
 *
 *	$Id: util.c,v 1.30 2012/03/10 02:23:39 conrads Exp $
 *
 *****************************************************************************/

#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/param.h>
#include <sys/stat.h>

/* only reference in this file to this next set of pointers is in cleanup(),
	where they're freed just before the program exits, so skip the comments
	for each one :-)
 */
extern	char   *index_buf;
extern	char   *template_buf_category;
extern	char   *template_buf_port;
extern	char   *template_buf_top;
extern	char *(*index_categories)[];

extern	char	ports_dir_plus[PATH_MAX];	/* ports_dir + trailing slash */

extern	int		normal_paths;		/* make relative paths relative to current
										working directory instead of ports_dir
									 */
extern	int		verbose;			/* increase the amount of output */

/* prototypes for global functions */

char   *load_file(const char *path);
char   *resolve_path(const char *path);
char   *skip_leading(char *str, const char *pattern);
int		validate_path(const char *path);
void	cleanup(void);

/*****************************************************************************/

void cleanup(void)
/*
	final cleanup routine, to release all of the memory we allocated

	registered with atexit(3) during initialization, to be called
	automatically when program exits
*/
{
	if (index_buf != NULL)
		free(index_buf);

	if (template_buf_top != NULL)
		free(template_buf_top);

	if (template_buf_category != NULL)
		free(template_buf_category);

	if (template_buf_port != NULL)
		free(template_buf_port);

	if (index_categories != NULL)
		free(index_categories);
}

/*****************************************************************************/

char *load_file(const char *path)
/*
	load the file named by 'path' into a dynamically allocated buffer
	(used for the index and template files)
	
	buffer will have an extra nul character appended to mark end-of-file

	returns a pointer to the buffer on success, or NULL on failure
 */
{
	char   *buffer;		/* pointer to memory obtained via malloc(3) */
	int		fd;			/* file descriptor for open() and close() */
	size_t	size;		/* file's size, as returned by stat(2) */
	struct	stat sb;	/* will hold the results from stat(2) */

	/* get the file's size */
	if (stat(path, &sb) == -1)
	{
		perror(path);
		return NULL;
	}

	/* cast to squelch compiler warning */
	size = (unsigned) sb.st_size;

	/* allocate (size + 1) bytes, so we can mark end of file
		with an extra nul character
	 */
	if ((buffer = malloc(size + 1)) == NULL)
	{
		perror("malloc failure");
		return NULL;
	}

	/* display an info blurb at verbosity levels >= 2) */
	if (verbose > 1)
		fprintf(stderr, "Loading %s\n", path);

	/* open and read the entire file into memory */
	if ((fd = open(path, O_RDONLY|O_DIRECT)) == -1)
	{
		perror(path);
		return NULL;
	}

	/* another cast to squelch compiler warning */
	if (read(fd, buffer, size) != (ssize_t) size)
	{
		perror(path);
		close(fd);
		return NULL;
	}

	close(fd);

	/* add an extra nul to mark end of buffer */
	buffer[size] = '\0';

	return buffer;
}

/*****************************************************************************/

char *skip_leading(char *str, const char *pattern)
/*
	skip over 'pattern' at the beginning of 'str'

	returns	a pointer to the location in 'str' immediately following 'pattern',
	or 'str' if it does not begin with 'pattern'
	
	(adding the 'const' qualifier to str is just more trouble than it's worth;
	all it does is cause a couple of compiler warnings below; not like we're
	gonna be messing it up anyway, so why bother?)
 */
{
	size_t	len = strlen(pattern);

	return (strncmp(str, pattern, len) == 0) ? (str + len) : str;
}

/******************************************************************************/

int validate_path(const char *path)
/*
	verify that path exists and is a directory

	returns 0 on success, or -1
 */
{
	struct	stat sb;				/* information returned from stat() */

	/* call stat to get info for path */
	if (stat(path, &sb) == -1)
	{
		perror(path);
		return -1;
	}

	/* make sure it's a directory */
	if (!S_ISDIR(sb.st_mode))
	{
		fprintf(stderr, "%s is not a directory\n", path);
		return -1;
	}

	return 0;
}

/******************************************************************************/

char *resolve_path(const char *path)
/*
	Resolve the pathname pointed to by 'path' to a fully qualified pathname

	As a convenience, let any relative path be relative to ports_dir,
	unless the user selected the "-n" option, in which case path will be
	relative to the current working directory

	Returns a pointer to the fully qualified pathname returned by realpath(3),
	or NULL on error

	Uses realpath(3) to do the grunt work

	Caller should free() the returned pointer after it is no longer needed
*/
{
	char *real_path;			/* result from realpath(3) */
	char pathbuf[PATH_MAX];		/* temporary buffer */

	if (*path == '/')			/* absolute path, use "as is" (after running it
									through realpath() below
								 */
	{
		strcpy(pathbuf, path);
	}
	else if (*path == '.')		/* pathnames beginning with "." or ".." are
									always relative to the current working
									directory, regardless of the setting of the
									normal_paths flag
								 */
	{
		if (getcwd(pathbuf, (size_t)PATH_MAX) == NULL)
		{
			perror("getcwd");
			return NULL;
		}
		else
		{
			/* append path to current working directory in pathbuf */
			strcat(pathbuf, "/");
			strcat(pathbuf, path);
		}
	}
	else						/* relative path */
	{
		if (!normal_paths)		/* append path to ports_dir/ in pathbuf */
		{
			strcpy(pathbuf, ports_dir_plus);
			strcat(pathbuf, path);
		}
		else					/* append path to current working directory */
		{
			if (getcwd(pathbuf, (size_t)PATH_MAX) == NULL)
			{
				perror("getcwd");
				return NULL;
			}
			else
			{
				/* append path to current working directory in pathbuf */
				strcat(pathbuf, "/");
				strcat(pathbuf, path);
			}
		}
	}

	/* Now run what we have through realpath() to "smooth it out"
		Note: paths returned by realpath will *never* have a trailing slash
	 */
	if ((real_path = realpath(pathbuf, NULL)) == NULL)
	{
		perror(pathbuf);
		return NULL;
	}

	/* it's still possible that the last element of real_path doesn't actually
		exist (see realpath(3)), so do one more check, so that we'll produce
		consistent results for any non-existent paths ("no such file or
		directory"), otherwise the path may be passed along for further
		processing, which we don't want to happen
	 */
	if (validate_path(real_path) != 0)
		return NULL;

	return real_path;
}

==> mkreadmes-1.3.tar.bz2: mkreadmes-1.3/util.c <==
 /*
 *	util.c
 *
 *	utility routines called from a number of different places
 *
 *	$Id: util.c,v 1.41 2012/03/27 07:19:27 conrads Exp $
 *
 *****************************************************************************/

#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/param.h>
#include <sys/stat.h>

/* only reference in this file to this next set of pointers is in cleanup(),
	where they're freed just before the program exits, so skip the comments
	for each one :-)
 */
extern	char   *index_buf;
extern	char   *template_buf_category;
extern	char   *template_buf_port;
extern	char   *template_buf_top;
extern	char *(*index_categories)[];

extern	char	ports_dir_plus[PATH_MAX];	/* ports_dir + trailing slash */

extern	int		normal_paths;		/* make relative paths relative to current
										working directory instead of ports_dir
									 */
extern	int		verbose;			/* increase the amount of output */

/* prototypes for global functions */

char   *load_file(const char *path);
char   *resolve_path(const char *path);
char   *skip_leading(char *str, const char *pattern);
int		validate_path(const char *path);
void	cleanup(void);

/*****************************************************************************/

void cleanup(void)
/*
	final cleanup routine, to release all of the memory we allocated

	registered with atexit(3) during initialization, to be called
	automatically when program exits
*/
{
	if (index_buf != NULL)
		free(index_buf);

	if (template_buf_top != NULL)
		free(template_buf_top);

	if (template_buf_category != NULL)
		free(template_buf_category);

	if (template_buf_port != NULL)
		free(template_buf_port);

	if (index_categories != NULL)
		free(index_categories);
}

/*****************************************************************************/

char *load_file(const char *path)
/*
	load the file named by 'path' into a dynamically allocated buffer
	(used for the index and template files)

	buffer will have an extra nul character appended to mark end-of-file

	returns a pointer to the buffer on success, or NULL on failure
 */
{
	char   *buffer;		/* pointer to memory obtained via malloc(3) */
	int		fd;			/* file descriptor for open() and close() */
	size_t	size;		/* file's size, as returned by stat(2) */
	struct	stat sb;	/* will hold the results from stat(2) */

	/* get the file's size */
	if (stat(path, &sb) == -1)
	{
		perror(path);
		return NULL;
	}

	/* cast to squelch compiler warning */
	size = (unsigned) sb.st_size;

	/* allocate (size + 1) bytes, so we can mark end of file
		with an extra nul character
	 */
	if ((buffer = malloc(size + 1)) == NULL)
	{
		perror("malloc failure");
		return NULL;
	}

	/* display an info blurb at verbosity levels >= 2) */
	if (verbose > 1)
		fprintf(stderr, "Loading %s\n", path);

	/* open and read the entire file into memory */
	if ((fd = open(path, O_RDONLY|O_DIRECT)) == -1)
	{
		perror(path);
		return NULL;
	}

	/* another cast to squelch compiler warning */
	if (read(fd, buffer, size) != (ssize_t) size)
	{
		perror(path);
		close(fd);
		return NULL;
	}

	close(fd);

	/* add an extra nul to mark end of buffer */
	buffer[size] = '\0';

	return buffer;
}

/*****************************************************************************/

char *skip_leading(char *str, const char *pattern)
/*
	skip over 'pattern' at the beginning of 'str'

	returns a pointer to the location in 'str' immediately following 'pattern',
	or 'str' if it does not begin with 'pattern'.  if the two strings are
	exactly equal, returns a pointer to the nul end-of-string marker for 'str'

	(adding the 'const' qualifier to str is just more trouble than it's worth;
	all it does is cause a couple of compiler warnings below; not like we're
	gonna be messing it up anyway, so why bother?)
 */
{
    char *ptr = str;

	while (*pattern)
	{
		if (*ptr != *pattern)
			break;
		++ptr;
		++pattern;
	}

	/* (*pattern != '\0') means no match */
	return (*pattern) ? str : ptr;
}

/******************************************************************************/

int validate_path(const char *path)
/*
	verify that path exists and is a directory

	returns 0 on success, or -1
 */
{
	struct	stat sb;				/* information returned from stat() */

	/* call stat to get info for path */
	if (stat(path, &sb) == -1)
	{
		perror(path);
		return -1;
	}

	/* make sure it's a directory */
	if (!S_ISDIR(sb.st_mode))
	{
		fprintf(stderr, "%s is not a directory\n", path);
		return -1;
	}

	return 0;
}

/******************************************************************************/

char *resolve_path(const char *path)
/*
	Resolve the pathname pointed to by 'path' to a fully qualified pathname

	As a convenience, let any relative path be relative to ports_dir,
	unless the user selected the "-n" option, in which case path will be
	relative to the current working directory

	Returns a pointer to the fully qualified pathname returned by realpath(3),
	or NULL on error

	Uses realpath(3) to do the grunt work

	Caller should free() the returned pointer after it is no longer needed
*/
{
	char *real_path;			/* result from realpath(3) */
	char pathbuf[PATH_MAX];		/* temporary buffer */

	if (*path == '/')			/* absolute path, use "as is" (after running it
									through realpath() below
								 */
	{
		strcpy(pathbuf, path);
	}
	else if (*path == '.')		/* pathnames beginning with "." or ".." are
									always relative to the current working
									directory, regardless of the setting of the
									normal_paths flag
								 */
	{
		if (getcwd(pathbuf, (size_t)PATH_MAX) == NULL)
		{
			perror("getcwd");
			return NULL;
		}
		else
		{
			/* append path to current working directory in pathbuf */
			strcat(pathbuf, "/");
			strcat(pathbuf, path);
		}
	}
	else						/* relative path */
	{
		if (!normal_paths)		/* append path to ports_dir/ in pathbuf */
		{
			strcpy(pathbuf, ports_dir_plus);
			strcat(pathbuf, path);
		}
		else					/* append path to current working directory */
		{
			if (getcwd(pathbuf, (size_t)PATH_MAX) == NULL)
			{
				perror("getcwd");
				return NULL;
			}
			else
			{
				/* append path to current working directory in pathbuf */
				strcat(pathbuf, "/");
				strcat(pathbuf, path);
			}
		}
	}

	/* Now run what we have through realpath() to "smooth it out"
		Note: paths returned by realpath will *never* have a trailing slash
	 */
	if ((real_path = realpath(pathbuf, NULL)) == NULL)
	{
		perror(pathbuf);
		return NULL;
	}

	/* it's still possible that the last element of real_path doesn't actually
		exist (see realpath(3)), so do one more check, so that we'll produce
		consistent results for any non-existent paths ("no such file or
		directory"), otherwise the path may be passed along for further
		processing, which we don't want to happen
	 */
	if (validate_path(real_path) != 0)
		return NULL;

	return real_path;
}
